"""backup.py with make_tarfile, local_tarfile_copy, remote_tarfile_copy takes backup jenkins_home"""
import os
import shutil
import tarfile

import boto3


def make_tarfile(archive_name, jenkins_home):
    """make_tarfile(archive_name, jenkins_home) will tar the jenkins_home directory"""
    with tarfile.open(archive_name, "w:gz") as tar:
        tar.add(jenkins_home, arcname=os.path.basename(jenkins_home))


def local_tarfile_copy(archive_name, dest_location):
    """local_tarfile_copy is to move a tar from one location to another location"""
    shutil.move(archive_name, dest_location)


def remote_tarfile_copy(bucket_name, archive_name):
    """remote_tarfile_copy move the tar file from a local dir to an s3 bucket. For this
    method to work, the AWS credentials should be preset either in the terminal session
    or in the ~/.aws/credentials file from the machine you are running the jenkins-backup-restore-cli from."""
    s3_resource = boto3.resource('s3')
    s3_resource.Object(bucket_name, archive_name).upload_file(Filename=archive_name)
    os.remove(archive_name)



import click
import datetime
from pyfiglet import Figlet
from .jenkins.backup import make_tarfile, local_tarfile_copy, remote_tarfile_copy
from .jenkins.restore import local_tarfile, remote_tarfile

date = datetime.date.today()

tool_name = Figlet(font='big')
print(tool_name.renderText('Jenkins Backup Restore Cli'))


@click.group()
@click.version_option(version='1.0.7')
@click.option('--custom-archive-name', default='jenkins-backup-DATE.tar.gz', help='Custom name for the jenkins backup')
@click.option('--jenkins-home-dir', help='Custom Jenkins home directory other than the default (/var/jenkins_home)', default='/var/jenkins_home')
@click.pass_context
def jenkins_backup_restore_cli(ctx, custom_archive_name, jenkins_home_dir):

    if custom_archive_name:
        archive_name = custom_archive_name
    else:
        archive_name = f"jenkins-backup-{date}.tar.gz"

    if jenkins_home_dir:
        jenkins_home = jenkins_home_dir

    ctx.obj['archive_name'] = archive_name
    ctx.obj['jenkins_home'] = jenkins_home


@jenkins_backup_restore_cli.command()
@click.option('--bd', '--backup-destination-path', help='Local path to store the backup')
@click.pass_context
def backup_local(ctx, backup_destination_path):
    make_tarfile(ctx.obj['archive_name'], ctx.obj['jenkins_home'])
    local_tarfile_copy(ctx.obj['archive_name'], backup_destination_path)


@jenkins_backup_restore_cli.command()
@click.option('--bb', '--backup-bucket-name', help='Bucket name to push the backup tar file to s3')
@click.pass_context
def backup_s3(ctx, backup_bucket_name):
    make_tarfile(ctx.obj['archive_name'], ctx.obj['jenkins_home'])
    remote_tarfile_copy(backup_bucket_name, ctx.obj['archive_name'])


@jenkins_backup_restore_cli.command()
@click.option('--rs', '--restore-source-path', help='Path to the archive in local directory')
@click.option('--rd', '--restore-path', help='Use the flag if jenkins_home location is other than the default path (/var/jenkins_home)', default='/var/jenkins_home')
@click.pass_context
def restore_local(ctx, restore_source_path, restore_path):
    local_tarfile(ctx.obj['archive_name'], restore_source_path, restore_path)


@jenkins_backup_restore_cli.command()
@click.option('--rb', '--restore-bucket-name', help='Bucket name to download the backup tar from')
@click.option('--adp', '--artifact-destination-path', help='Path to save the downloaded archive from an s3 bucket')
@click.option('--rd', '--restore-path', help='Use the flag if destination path is other than the default path (/var/jenkins_home)', default='/var/jenkins_home')
@click.pass_context
def restore_s3(ctx, restore_bucket_name, artifact_destination_path, restore_path):
    remote_tarfile(restore_bucket_name, ctx.obj['archive_name'], artifact_destination_path, restore_path)


jenkins_backup_restore_cli.add_command(backup_local)
jenkins_backup_restore_cli.add_command(backup_s3)
jenkins_backup_restore_cli.add_command(restore_local)
jenkins_backup_restore_cli.add_command(restore_s3)

if __name__ == '__main__':
    jenkins_backup_restore_cli(obj={})


"""restore.py with below local_tarfile and remote_tarfile methods restores the jenkins backup"""
import tarfile
import os
import shutil
import boto3
import botocore


def local_tarfile(archive_name, archive_path, jenkins_home):
    """local_tarfile extracts a tar file in a user specified location"""
    my_tar = tarfile.open(f'{archive_path}/{archive_name}')
    my_tar.extractall(jenkins_home)
    my_tar.close()


def remote_tarfile(bucket_name, archive_name, dest_path, jenkins_home):
    """remote_tarfile not only pulls the tar from the s3 bucket but also extracts the
    tar file in the user specified location, For this method to work, the AWS credentials
    should be preset in the terminal from where ever you are running the
    jenkins-backup-restore-cli from."""
    s3_resource = boto3.client('s3')
    try:
        s3_resource.download_file(bucket_name, archive_name, f'{dest_path}/{archive_name}')
    except botocore.exceptions.ClientError as err:
        if err.response['Error']['Code'] == "404":
            print("The object does not exist.")
        else:
            raise
    extract_dir = jenkins_home.replace('jenkins_home', '')
    shutil.unpack_archive(f'{dest_path}/{archive_name}', extract_dir)
    os.remove(f'{dest_path}/{archive_name}')



from os import path

from setuptools import setup, find_packages
import os
import io


def read(reqpath, encoding='utf-8'):
    reqpath = os.path.join(os.path.dirname(__file__), reqpath)
    with io.open(reqpath, encoding=encoding) as fp:
        return fp.read()


def get_install_requirements(reqpath):
    content = read(reqpath)
    return [
        req
        for req in content.split("\n")
        if req != '' and not req.startswith('#')
    ]


INSTALL_REQUIRES = get_install_requirements("requirements.txt")


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='jenkins-backup-restore-cli',
    version='1.0.7',
    author='Surya Lolla',
    author_email='suryasaicharan93@gmail.com',
    description='A Jenkins backup and restore python cli tool with arguments.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/venkatalolla/jenkins-backup-restore.git',
    license='MIT',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'jenkins-backup-restore-cli = main.jenkins_backup_restore_cli:jenkins_backup_restore_cli'
        ]
    },
    classifiers=(
        "Development Status :: 3 - Alpha",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Topic :: System :: Systems Administration",
        "Topic :: System :: Recovery Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
    ),
    keywords='Jenkins backup and restore cli',
    install_requires=INSTALL_REQUIRES,
    zip_safe=False
)
